// $Id: TCPEncoding.java,v 1.4 2001/05/27 21:15:21 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.net;


/**
 * TCP protocol encoding information.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.4 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/05/27 21:15:21 $
 */
public class TCPEncoding
{
  /**
   * TCPPacket factory.
   */
  public static TCPPacket create(int lLen, byte [] bytes) {
    String msa = EthernetEncoding.extractMACAddress(lLen, true, bytes); 
    String mda = EthernetEncoding.extractMACAddress(lLen, false, bytes); 

    String sa = IPEncoding.extractAddress(true, lLen, bytes);
    String da = IPEncoding.extractAddress(false, lLen, bytes);

    int sp = TCPEncoding.extractTCPSrcPort(lLen, bytes);
    int dp = TCPEncoding.extractTCPDstPort(lLen, bytes);

    int ipProtocol = IPEncoding.extractProtocol(lLen, bytes);

    return new TCPPacket(lLen, bytes, msa, mda, sa, sp, da, dp, ipProtocol);
  }

  /**
   * Extract the source port from a tcp packet.
   * @param llen the length of the link-level header.
   * @param packetBytes packet bytes, including the link-layer and IP headers.
   */
  public static int extractTCPSrcPort(int lLen, byte [] packetBytes) {
    return 
      (0xff & packetBytes[lLen + SP_POS]) << 8 | 
      (0xff & packetBytes[lLen + SP_POS + 1]);
  }

  /**
   * Extract the destination port from a tcp packet.
   * @param llen the length of the link-level header.
   * @param packetBytes packet bytes, including the link-layer and IP headers.
   */
  public static int extractTCPDstPort(int lLen, byte [] packetBytes) {
    return 
      (0xff & packetBytes[lLen + DP_POS]) << 8 | 
      (0xff & packetBytes[lLen + DP_POS + 1]);
  }


  /**
   * Position of the source port within a TCP header.
   */
  private final static int SP_POS = IPEncoding.HEADER_LEN;

  /**
   * Position of the destination port within a TCP header.
   */
  private final static int DP_POS = IPEncoding.HEADER_LEN + 2;

  /**
   * Length in bytes of a TCP header.
   */
  public final static int HEADER_LEN = 20;

  private String _rcsid = 
    "$Id: TCPEncoding.java,v 1.4 2001/05/27 21:15:21 pcharles Exp $";
}
